#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fslib_1 = require("@yarnpkg/fslib");
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const dynamicRequire_1 = require("./dynamicRequire");
const generateSdk_1 = require("./generateSdk");
const [, , name, ...rest] = process.argv;
if (name === `--help` || name === `-h`)
    help(false);
else if (name === `--sdk` && rest.length === 0)
    sdk(process.cwd());
else if (name === `--sdk` && rest.length === 1)
    sdk(rest[0]);
else if (typeof name !== `undefined` && name[0] !== `-`)
    run(name, rest);
else
    help(true);
function help(error) {
    const logFn = error ? console.error : console.log;
    process.exitCode = error ? 1 : 0;
    logFn(`Usage: yarn pnpify --sdk [path?]`);
    logFn(`Usage: yarn pnpify <program> [...argv]`);
    logFn();
    logFn(`Setups a TypeScript sdk for use within your VSCode editor instance`);
    logFn(`More info at https://yarnpkg.com/advanced/pnpify`);
}
function sdk(cwd) {
    let nextProjectRoot = fslib_1.npath.toPortablePath(cwd);
    let currProjectRoot = null;
    let isCJS = '';
    while (nextProjectRoot !== currProjectRoot) {
        currProjectRoot = nextProjectRoot;
        nextProjectRoot = fslib_1.ppath.dirname(currProjectRoot);
        if (fslib_1.xfs.existsSync(fslib_1.ppath.join(currProjectRoot, `.pnp.js`)))
            break;
        if (fslib_1.xfs.existsSync(fslib_1.ppath.join(currProjectRoot, `.pnp.cjs`))) {
            isCJS = 'c';
            break;
        }
    }
    if (nextProjectRoot === currProjectRoot)
        throw new Error(`This tool can only be used with projects using Yarn Plug'n'Play`);
    const pnpPath = fslib_1.ppath.join(currProjectRoot, `.pnp.${isCJS}js`);
    const pnpApi = dynamicRequire_1.dynamicRequire(pnpPath);
    generateSdk_1.generateSdk(pnpApi).catch(error => {
        console.error(error.stack);
        process.exitCode = 1;
    });
}
function run(name, argv) {
    let { NODE_OPTIONS } = process.env;
    NODE_OPTIONS = `${NODE_OPTIONS || ``} --require ${dynamicRequire_1.dynamicRequire.resolve(`@yarnpkg/pnpify`)}`.trim();
    const child = cross_spawn_1.default(name, argv, {
        env: Object.assign(Object.assign({}, process.env), { NODE_OPTIONS }),
        stdio: `inherit`,
    });
    child.on(`exit`, code => {
        process.exitCode = code !== null ? code : 1;
    });
}
